@@ -1,29 +1,36 @@
 
 
-
 as_integer_tensor <- function(x, dtype = tf$int64) {
+
   # recurse over lists
   if (is.list(x) || (is.numeric(x) && length(x) > 1))
     lapply(x, function(elem) as_integer_tensor(elem, dtype))
   else if (is.null(x))
     x
   else if (is_tensor(x))
-    tf$cast(x, dtype = dtype)
+    tensor_value(tf$cast(x, dtype = dtype))
   else
-    tf$constant(as.integer(x), dtype = dtype)
+    tensor_value(tf$constant(as.integer(x), dtype = dtype))
 }
 
 as_tensor_shapes <- function(x) {
   if (is.list(x))
     tuple(lapply(x, as_tensor_shapes))
   else if (is_tensor(x))
-    tf$cast(x, dtype = tf$int64)
-  else if (inherits("x", "python.builtin.object"))
+    tensor_value(tf$cast(x, dtype = tf$int64))
+  else if (inherits(x, "python.builtin.object"))
     x
   else if (is.null(x))
-    tf$constant(-1L, dtype = tf$int64)
+    tensor_value(tf$constant(-1L, dtype = tf$int64))
   else
-    tf$constant(as.integer(x), dtype = tf$int64)
+    tensor_value(tf$constant(as.integer(x), dtype = tf$int64))
+}
+
+tensor_value <- function(x) {
+  if (is_eager_tensor(x))
+    x$numpy()
+  else
+    x
 }
 
 
@@ -38,12 +45,15 @@ with_session <- function(f, session = NULL) {
 }
 
 
-validate_tf_version <- function() {
+validate_tf_version <- function(required_ver = "1.4", feature_name = "tfdatasets") {
   tf_ver <- tensorflow::tf_version()
-  required_ver <- "1.4"
-  if (tf_ver < required_ver) {
+  if (is.null(tf_ver)) {
+    stop("You need to install TensorFlow to use tfdatasets ",
+         "-- install with tensorflow::install_tensorflow()",
+         call. = FALSE)
+  } else if (tf_ver < required_ver) {
     stop(
-      "tfdatasets requires version ", required_ver, " ",
+      feature_name, " requires version ", required_ver, " ",
       "of TensorFlow (you are currently running version ", tf_ver, ").",
       call. = FALSE
     )
@@ -64,5 +74,10 @@ is_tensor <- function(x) {
   inherits(x, "tensorflow.python.framework.ops.Tensor")
 }
 
+is_eager_tensor <- function(x) {
+  inherits(x, "python.builtin.EagerTensor") ||
+  inherits(x, "tensorflow.python.framework.ops.EagerTensor")
+}
+
 
 
